#include "GeometryHelper.h"
#include <QTextStream>
#include <QFile>

void WriteVertex(QTextStream& stream, double x, double y, double z){
    stream << "v " << x << " " << y << " " << z << "\n";
}

void WriteFace(QTextStream& stream,int v1, int v2, int v3){
    stream << "f " << v1 << " " << v2 << " " << v3 << "\n";
}

bool GeometryHelper::ExportBuildingObjFile(QString filePath, double length, double width, double height)
{
    //Opening the file for writing
    QFile ObjFile(filePath);
    if (!ObjFile.open(QIODevice::WriteOnly | QIODevice::Text))
            return false;

    //Writing header
    ObjFile.write("#Building Geometry generated by WE-UQ\n");
    ObjFile.write("o Building\n");

    QTextStream stream(&ObjFile);

    //Writing Vertices in mm
    WriteVertex(stream, -length/2, width/2, 0.0);
    WriteVertex(stream, -length/2, -width/2, height);
    WriteVertex(stream, -length/2, width/2, height);
    WriteVertex(stream, -length/2, -width/2, 0.0);
    WriteVertex(stream, length/2, -width/2, height);
    WriteVertex(stream, length/2, width/2, 0.0);
    WriteVertex(stream, length/2, width/2, height);
    WriteVertex(stream, length/2, -width/2, 0.0);

    //Writing faces
    WriteFace(stream, 1, 2, 3);
    WriteFace(stream, 4, 2, 1);
    WriteFace(stream, 5, 6, 7);
    WriteFace(stream, 5, 8, 6);
    WriteFace(stream, 8, 5, 4);
    WriteFace(stream, 4, 5, 2);
    WriteFace(stream, 7, 6, 1);
    WriteFace(stream, 7, 1, 3);
    WriteFace(stream, 1, 6, 4);
    WriteFace(stream, 4, 6, 8);
    WriteFace(stream, 7, 3, 2);
    WriteFace(stream, 7, 2, 5);

    ObjFile.close();

    return true;
}
